#ifndef _SerialPort_SerialPort_h
#define _SerialPort_SerialPort_h



#include <Core/Core.h>
#include <termios.h>



class SerialPort
{
public:
	typedef SerialPort CLASSNAME;

	class SerialException : public Upp::Exc
	{
	public:
		typedef SerialException CLASSNAME;

		/**
		 * @brief     Constructeur d'une Exception (1er niveau : a l'origine)
		 *
		 * @param[in] msg : description de l'erreur
		 */
		SerialException(const Upp::String& msg) :
			Upp::Exc(msg)
			{
			}
		
		SerialException(const Upp::String& msg, const Upp::Exc& cause)
		: Exc( msg + "\n Caused by >   " + cause )
		{
		}

		virtual ~SerialException() throw()
		{
		}
	};
	
	public:
		typedef enum {
			 BDS_115200 = B115200
			,BDS_57600 = B57600
			,BDS_38400 = B38400
			,BDS_19200 = B19200
			,BDS_9600 = B9600
			,BDS_4800 = B4800
			,BDS_2400 = B2400
			,BDS_1200 = B1200
			,BDS_600 = B600
			,BDS_300 = B300
			,BDS_110 = B110
		} SerialPortSpeeds;
		
		// Creation ET ouverture du port serie
		SerialPort();
		
		void Open(const char * const device, const SerialPortSpeeds sp );
		void Close();
		
		// destruction ET fermeture du pôrt serie
		virtual ~SerialPort();
		
		
		typedef enum {
			  WRITE_OK = 0
			, WRITE_ERROR = -1
		} WriteReturnValues;

		///< envoie des octets
		WriteReturnValues write( int nbBytesToWrite, const char* const buf );
		
		
		typedef enum {
			  READ_OK=0
			, READ_TIMEDOUT = 1
			, READ_ERROR = -1
		} ReadReturnValues;
		
		// lecture d'octets
		ReadReturnValues read(char* buf, int& nbBytesToRead, int timeOutms=-1);
		
		// Debug method
		static Upp::String BufToString(const char * const buf, const int nbBytes, const char * const prefixString);

	private:
		int    m_fileDesc;
		Upp::String m_deviceName;
};


#endif

